#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import atexit
import logging
logger = logging.getLogger('PyAMS (content.es)')

import sys

# import interfaces
from pyams_content_es.interfaces import INDEXER_HANDLER_KEY
from pyramid.interfaces import IApplicationCreated

# import packages
from pyams_content_es.process import ContentIndexerProcess, ContentIndexerMessageHandler
from pyams_zmq.process import process_exit_func
from pyramid.events import subscriber
from zope.component.globalregistry import getGlobalSiteManager


def include_package(config):
    """Pyramid include"""

    # add translations
    config.add_translation_dirs('pyams_content_es:locales')

    # load registry components
    try:
        import pyams_zmi
    except ImportError:
        config.scan(ignore='pyams_content_es.zmi')
    else:
        config.scan()


@subscriber(IApplicationCreated)
def handle_new_application(event):
    """Start indexer process when application created"""

    # check for upgrade mode
    if sys.argv[0].endswith('pyams_upgrade'):
        return

    registry = getGlobalSiteManager()
    settings = registry.settings
    start_handler = settings.get(INDEXER_HANDLER_KEY, False)
    if start_handler:
        process = None
        # create content indexer process
        try:
            process = ContentIndexerProcess(start_handler, ContentIndexerMessageHandler, registry)
            logger.debug('Starting content indexer process {0!r}...'.format(process))
            process.start()
            if process.is_alive():
                atexit.register(process_exit_func, process=process)
        finally:
            if process and not process.is_alive():
                process.terminate()
                process.join()
